#!/usr/bin/python
import yaml

from urlparse import urlparse
from base64 import b64encode
from itertools import chain

from charmhelpers.core import hookenv
from charmhelpers.core.services import helpers


class ApacheVhostRelation(helpers.RelationContext):
    name = 'webservice'
    interface = 'webservice'
    required_keys = ['services']

    def get_data(self):
        """ After we have our relation data, we activate it into live Python
        data structures, including urlparsing the urls in all the services. """

        super(ApacheVhostRelation, self).get_data()
        hookenv.log(str(self))

        if not hookenv.relation_ids(self.name):
            return

        for unit in self[self.name]:
            if hasattr(unit['services'], 'capitalize'):
                # It's a STRIIIIIING!
                unit['services'] = yaml.safe_load(unit['services'])
            for service in unit['services']:
                if not hasattr(service['url'], 'geturl'):
                    service['url'] = urlparse(service['url'])
        hookenv.log(str(self))

    def get_ports(self):
        if not self:
            return set()
        ports = set()
        services = chain.from_iterable([x["services"] for x in self["webservice"]])
        for service in services:
            if service["url"].port:
                ports.update((service["url"].port,))
            elif service["url"].scheme == "https":
               ports.update((443,))
            else:
               ports.update((80,))
        return ports


class LogArchiveRelation(helpers.RelationContext):
    name = 'log-archive'
    interface = 'log-archive'

    def provide_data(self):
        """log-archive relation needs base64 encoded directory names."""
        return {'logdirs': b64encode('/var/log/apache2'), 'archive_after': '1'}

class MpmConfig(dict):
    """ If we have auto tuning set, we want to calculate these values. If not,
    use what's specified in config.yaml """


    def __init__(self):
        config = hookenv.config()
        if config['mpm_auto_tuning']:
            if config['mpm_type'] == 'worker':
                mpm_start_servers = 5
                mpm_max_clients = 150
                mpm_max_requests_per_child = 0
            else:
                mpm_start_servers = 5
                mpm_max_clients = 1024
                mpm_max_requests_per_child = 400

            # Prefork Only
            mpm_min_spare_servers = 5
            mpm_max_spare_servers = 10
            mpm_server_limit = 128

            # Worker Only
            mpm_min_spare_threads = 25
            mpm_max_spare_threads = 75
            mpm_thread_limit = 64
            mpm_threads_per_child = 64
        else:
            mpm_start_servers = config['mpm_start_servers']
            mpm_server_limit = config['mpm_server_limit']
            mpm_max_clients = config['mpm_max_clients']
            mpm_max_requests_per_child = config['mpm_max_requests_per_child']
            mpm_min_spare_servers = config['mpm_min_spare_servers']
            mpm_max_spare_servers = config['mpm_max_spare_servers']
            mpm_min_spare_threads = config['mpm_min_spare_threads']
            mpm_max_spare_threads = config['mpm_max_spare_threads']
            mpm_thread_limit = config['mpm_thread_limit']
            mpm_threads_per_child = config['mpm_threads_per_child']
        self['mpm_config'] = {
            'mpm_start_servers': mpm_start_servers,
            'mpm_server_limit': mpm_server_limit,
            'mpm_max_clients': mpm_max_clients,
            'mpm_max_requests_per_child': mpm_max_requests_per_child,
            'mpm_min_spare_servers': mpm_min_spare_servers,
            'mpm_max_spare_servers': mpm_max_spare_servers,
            'mpm_min_spare_threads': mpm_min_spare_threads,
            'mpm_max_spare_threads': mpm_max_spare_threads,
            'mpm_thread_limit': mpm_thread_limit,
            'mpm_threads_per_child': mpm_threads_per_child,
        }
