#!/usr/bin/python
from charmhelpers.core.services.base import ServiceManager
from charmhelpers.core.services import helpers

import actions
import apache_helpers


def manage():
    manager = ServiceManager([
        {
            'service': 'apache2',
            'ports': [80, 443],
            'required_data': [
                helpers.RequiredConfig(),
                apache_helpers.MpmConfig(),
            ],
            'data_ready': [
                actions.install_apache,
                helpers.render_template(
                    source='apache-apparmor.j2',
                    target='/etc/apparmor.d/webserver.d/apache2',
                ),
                helpers.render_template(
                    source='apache-logrotate.j2',
                    target='/etc/logrotate.d/apache2',
                ),
                helpers.render_template(
                    source='mpm_prefork.conf.j2',
                    target='/etc/apache2/mods-available/mpm_prefork.conf',
                ),
                helpers.render_template(
                    source='mpm_worker.conf.j2',
                    target='/etc/apache2/mods-available/mpm_worker.conf',
                ),
            ],
        },
        {
            'service': 'apache-vhost',
            'required_data': [
                helpers.RequiredConfig(),
                apache_helpers.ApacheVhostRelation(),
                {
                    'cert_dir': '/etc/apache2/ssl/',
                    'ports': apache_helpers.ApacheVhostRelation().get_ports(),
                },
            ],
            'provided_data': [helpers.HttpRelation],
            'data_ready': [
                actions.install_extra_packages,
                actions.write_certs,
                helpers.render_template(
                    source='apache-vhost.j2',
                    target=actions.vhost_path,
                ),
                helpers.render_template(
                    source='no-portsconf.j2',
                    target='/etc/apache2/ports.conf',
                ),
                helpers.render_template(
                    source='apache-security.j2',
                    target='/etc/apache2/conf.d/security',
                ),
                actions.enable_vhost,
                actions.enable_disable_options,
            ],
            'start': [actions.vhost_ready],
            'data_lost': [
                helpers.render_template(
                    source='apache-vhost.j2',
                    target=actions.vhost_path,
                ),
                actions.enable_vhost,
                actions.enable_disable_options,
            ],
        },
        {
            'service': 'log-archive',
            'required_data': [apache_helpers.LogArchiveRelation()],
            'provided_data': [apache_helpers.LogArchiveRelation()],
        },
    ])
    manager.manage()
